#include "GlobalConfig.h"
#include <QSettings>

#include "spdlog/spdlog.h"


/* ====================================================================================
 * **************************    GlobalConfig成员函数    ******************************
 * ====================================================================================*/

/* 创建实例 */
GlobalConfig g_config;

GlobalConfig::GlobalConfig()
{
    ThreadSleepMS = 300;
}


/* 读取配置文件 */
bool GlobalConfig::readConfig(const QString& strConfigFile)
{
    if(strConfigFile.isEmpty())
    {
        SPDLOG_ERROR("读取配置文件失败! 配置文件名为空");
        return false;
    }
    SPDLOG_DEBUG("读取配置文件: {}", strConfigFile.toStdString());
    QSettings settings(strConfigFile, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");

    settings.beginGroup("System");

    AppUpdateOnWorkTimeInterval = settings.value("WorkOnInfoSecond", 600).toInt();  /* 更新在岗信息的时间间隔 */
    AppPeopleOnWork = settings.value("APPPEPOLEONWORK", 300).toInt();         /* 离岗时间 */
    Contraband = settings.value("APPBADTHING", 50).toInt();                 /* 违禁物品出现的时间 */
    AppBadMan = settings.value("APPBADMAN", 50).toInt();                     /* 非法入侵 */
    AppTired = settings.value("APPTIRED", 50).toInt();                       /* 疲劳检测时间 */
    AppPeopleCont = settings.value("APPPEPOLECONT", 50).toInt();             /* 人员聚集时间 */
    AppPlayPhone = settings.value("APPPLAYPHONE", 50).toInt();               /* 玩手机识别 */
    AppMouse = settings.value("APPMOUSE", 50).toInt();                       /* 手势识别 */
    AppMask = settings.value("APPMASK", 5).toInt();                         /* 戴口罩识别 */

    CheckSet = settings.value("CHECKSET", 300).toInt();                       /* 服务端多久检测一次配置 */
    EventTimeValid = settings.value("EventTimeValid", 300).toInt();           /* 事件时间有效期 */

    Key = settings.value("Key").toString().toStdString();                           /* Key */
    Secret = settings.value("Secret").toString().toStdString();                     /* Secret */

    settings.endGroup();

    if(Key.empty() || Secret.empty())
    {
        SPDLOG_ERROR("读取配置文件失败! Key或Secret为空");
        return false;
    }

    return true;
}


/* 打印读取到的值 */
void GlobalConfig::printValue()
{
    SPDLOG_INFO("APPPEPOLEONWORK:   {}", AppPeopleOnWork);
    SPDLOG_INFO("APPBADTHING:       {}", Contraband);
    SPDLOG_INFO("APPBADMAN:         {}", AppBadMan);
    SPDLOG_INFO("APPTIRED:          {}", AppTired);
    SPDLOG_INFO("APPPEOPLECONT:     {}", AppPeopleCont);
    SPDLOG_INFO("APPPLAYPHONE:      {}", AppPlayPhone);
    SPDLOG_INFO("APPMOUSE:          {}", AppMouse);
    SPDLOG_INFO("APPMASK:           {}", AppMask);
    SPDLOG_INFO("CHECKSET:          {}", CheckSet);
    SPDLOG_INFO("EventTimeValid:    {}", EventTimeValid);
}


/* ====================================================================================
 * ************************    GlobalThreadInfo成员函数    *****************************
 * ====================================================================================*/

/* 获取线程运行标志位 */
bool GlobalThreadInfo::getRunning() const
{
    return m_bRunning;
}

/* 设置线程运行标志位 */
void GlobalThreadInfo::setRunning(bool bRunning)
{
    m_bRunning = bRunning;
}
